import os

from kafka import KafkaConsumer


def get_consumer(topic):
    consumer = KafkaConsumer(topic,
                             bootstrap_servers=os.getenv(
                                 'KAFKA_SUBSCRIBER', 'localhost:9093'),
                             group_id=os.getenv(
                                 "NEW_RELIC_APP_NAME")
                             )

    return consumer


import requests


def get_future_assets():
    asset_list_raw = requests.get(
        'https://fapi.binance.com/fapi/v1/ticker/price').json()

    asset_list_full = []
    for item in asset_list_raw:
        if (item['symbol'][-4:] == 'USDT' or item['symbol'][-4:] == 'BUSD'):
            # print(item)
            asset_list_full.append(item["symbol"])

    return asset_list_raw



import copy
import datetime

import pandas as pd


def remove_weird_chars(results):
    # results_json = json.loads(json.dumps(results.to_dict(), default=str))
    results_json = results

    new_res = {}

    for key in results_json.keys():
        old_key = key
        new_key = old_key.replace('.', '_')
        new_key = new_key.replace(' ', '_')
        new_key = new_key.replace('%', 'pcent')
        new_key = new_key.replace('[', '')
        new_key = new_key.replace(']', '')
        new_key = new_key.replace('(', '')
        new_key = new_key.replace(')', '')
        new_key = new_key.replace('$', '')
        new_key = new_key.replace('&', 'and')
        new_key = new_key.replace('#', 'n')
        new_key = new_key.replace('__', '_')
        new_key = new_key.lower()

        new_res[new_key] = results_json[old_key]

    return new_res

def dict_time_convert(stats):
    ret_res = {}
    
    for key in stats.keys():
        if isinstance(stats[key], pd.Timedelta):
            ret_res[key] = stats[key].total_seconds()
        elif isinstance(stats[key], pd.Timestamp):
            ret_res[key] = stats[key].to_pydatetime()
        else:
            ret_res[key] = stats[key]

    return ret_res

def result_maker(bt_stats, bt_params, bt_test_params, test_type):
    bt_stats = copy.deepcopy(bt_stats)
    bt_params = copy.deepcopy(bt_params)
    bt_test_params = copy.deepcopy(bt_test_params)
    test_type = copy.deepcopy(test_type)
                
    if '_trades' in bt_stats:
        del(bt_stats['_trades'])
    if '_strategy' in bt_stats:
        del(bt_stats['_strategy'])
    if '_equity_curve' in bt_stats:
        del(bt_stats['_equity_curve'])
        
    if '_id' in bt_test_params:
        del(bt_test_params["_id"])

    bt_stats = remove_weird_chars(bt_stats)
    bt_params = remove_weird_chars(bt_params)

    bt_stats = dict_time_convert(bt_stats)
    
    ret = {**bt_test_params, **bt_stats, **bt_params}
    ret['test_type'] = test_type
    ret['insert_timestamp'] = datetime.datetime.now()
        
    return ret


def list_maker(stats, test_params):
    list_doc = {}
    list_doc['insert_timestamp'] = datetime.datetime.now()
    list_doc['n_trades'] = stats['# Trades']
    list_doc['strategy'] = test_params['strategy']
    list_doc['period'] = test_params['period']
    list_doc['symbol'] = test_params['symbol']
    list_doc['trades_list'] = stats._trades.to_dict('records')
    
    td_list = []
    
    for item in list_doc['trades_list']:
        td_list.append(dict_time_convert(item))

    list_doc['trades_list'] = td_list
    ret = {**list_doc, **test_params}
    
    return ret



import os

from kafka import KafkaProducer


def get_producer():
    producer = KafkaProducer(bootstrap_servers=os.getenv(
                                 'KAFKA_SUBSCRIBER', 'localhost:9092'),
                             client_id=os.getenv(
                                 "NEW_RELIC_APP_NAME")
                             )

    return producer


import os

import pandas as pd
import pymongo


def get_client() -> pymongo.MongoClient:
    client = pymongo.MongoClient(
        os.getenv(
            'MONGO_URI', 'mongodb://root:QnjfRW7nl6@localhost:27017'),
        readPreference='secondaryPreferred',
        appname=os.getenv("NEW_RELIC_APP_NAME", "no-name-from-utils")
    )

    return client


def get_data(mongo_db, col_name, ticker, limit=999999999):

    client = get_client()
    db = client[mongo_db]
    history = db[col_name]

    results = history.find({'ticker': ticker},
                           sort=[('datetime', -1)]).limit(limit)
    results_list = list(results)

    if (len(results_list) == 0):
        return []

    data_df = pd.DataFrame(results_list)

    data_df = data_df.sort_values("datetime")

    data_df = data_df.rename(columns={"open": "Open",
                                      "high": "High",
                                      "low": "Low",
                                      "close": "Close"}
                             )

    data_df = data_df.set_index('datetime')

    return data_df


def post_results(mongo_db, symbol, test_period, doc, strategy):
    client = get_client()
    db = client[mongo_db]
    col = db['backtest_results']
    col.delete_one({"strategy": strategy,
                                                          "symbol": symbol,
                                                          "period": test_period
                                                          })
    col.update_one(
        {"strategy": strategy,
         "symbol": symbol,
         "period": test_period
         }, {"$set": doc}, upsert=True)
    return True


def post_list_results(mongo_db, symbol, test_period, doc, strategy):
    client = get_client()
    db = client[mongo_db]
    col = db['backtest_results_lists']
    col.delete_one({"strategy": strategy,
                                                                "symbol": symbol,
                                                                "period": test_period
                                                                })

    col.update_one(
        {"strategy": strategy,
         "symbol": symbol,
         "period": test_period
         }, {"$set": doc}, upsert=True)
    return True
